"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _stringify = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/json/stringify"));

var _bind = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/bind"));

var _embarkI18n = require("embark-i18n");

var _windowSize = _interopRequireDefault(require("window-size"));

function getWindowSize() {
  if (_windowSize.default) {
    return _windowSize.default.get();
  }

  return {
    width: 240,
    height: 75
  };
}

let Monitor = require('./monitor.js');

class Dashboard {
  constructor(options) {
    var _context, _context2;

    this.logger = options.logger;
    this.events = options.events;
    this.plugins = options.plugins;
    this.version = options.version;
    this.env = options.env;
    this.ipc = options.ipc;
    this.events.on('firstDeploymentDone', (0, _bind.default)(_context = this.checkWindowSize).call(_context, this));
    this.events.on('outputDone', (0, _bind.default)(_context2 = this.checkWindowSize).call(_context2, this));
  }

  checkWindowSize() {
    let size = getWindowSize();

    if (size.height < 40 || size.width < 118) {
      this.logger.warn((0, _embarkI18n.__)("tip: you can resize the terminal or disable the dashboard with") + " embark run --nodashboard".bold.underline);
    }
  }

  start(done) {
    var _context3, _context4;

    let monitor;
    monitor = new Monitor({
      env: this.env,
      events: this.events,
      version: this.version,
      ipc: this.ipc,
      logger: this.logger
    });
    this.logger.logFunction = monitor.logEntry;
    let plugin = this.plugins.createPlugin('dashboard', {});
    plugin.registerAPICall('ws', '/embark-api/dashboard', (ws, _req) => {
      let dashboardState = {
        contractsState: [],
        environment: "",
        status: "",
        availableServices: []
      };
      self.events.on('status', status => {
        dashboardState.status = status;
        ws.send((0, _stringify.default)(dashboardState));
      });
      self.events.on('servicesState', servicesState => {
        dashboardState.availableServices = servicesState;
        ws.send((0, _stringify.default)(dashboardState));
      });
    });
    this.events.on("deployment:contract:error", _contract => {
      this.events.request("contracts:state", (err, contracts) => {
        monitor.setContracts(contracts);
      });
    });
    this.events.on("deployment:contract:deployed", _contract => {
      this.events.request("contracts:state", (err, contracts) => {
        this.events.emit('contracts:state', contracts);
        monitor.setContracts(contracts);
      });
    });
    this.events.on("deployment:contract:undeployed", _contract => {
      this.events.request("contracts:state", (err, contracts) => {
        this.events.emit('contracts:state', contracts);
        monitor.setContracts(contracts);
      });
    });
    this.events.on('status', (0, _bind.default)(_context3 = monitor.setStatus).call(_context3, monitor));
    this.events.on('servicesState', (0, _bind.default)(_context4 = monitor.availableServices).call(_context4, monitor));
    this.logger.info('========================'.bold.green);
    this.logger.info(((0, _embarkI18n.__)('Welcome to Embark') + ' ' + this.version).yellow.bold);
    this.logger.info('========================'.bold.green);
    done();
  }

}

module.exports = Dashboard;
//# sourceMappingURL=dashboard.js.map