"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));

var _bind = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/bind"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

var _getOwnPropertyNames = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/get-own-property-names"));

var _sort = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/sort"));

var _lastIndexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/last-index-of"));

var _indexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/index-of"));

var _trim = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/trim"));

const repl = require("repl");

const util = require("util");

class REPL {
  constructor(options) {
    this.events = options.events;
    this.env = options.env;
    this.inputStream = options.inputStream || process.stdin;
    this.outputStream = options.outputStream || process.stdout;
    this.logText = options.logText;
    this.ipc = options.ipc;
    this.logger = options.logger;
    this.useDashboard = options.useDashboard;
  }

  addHistory(cmd) {
    const history = this.replServer.history;

    if (history[0] !== cmd) {
      history.unshift(cmd);
    }
  }

  enhancedEval(cmd, context, filename, callback) {
    if (this.useDashboard) {
      this.logger.consoleOnly("console> ".cyan + cmd.white);
    }

    this.events.request('console:executeCmd', (0, _trim.default)(cmd).call(cmd), function (err, message) {
      if (err) {
        // Do not return as the first param (error), because the dashboard doesn't print errors
        return callback(null, err.message ? err.message.red : err.red);
      }

      callback(null, message === undefined ? '' : message); // This way, we don't print undefined
    });
  }

  enhancedWriter(output) {
    if (typeof output === "string") {
      if (this.logText) this.logText.log(output);
      return output;
    }

    const inspectedOutput = util.inspect(output, {
      colors: true
    });
    if (this.logText) this.logText.log(inspectedOutput);
    return inspectedOutput;
  }

  complete(partial, cb) {
    // no function calls
    if ((0, _indexOf.default)(partial).call(partial, '(') !== -1) {
      return cb(null, [[], partial]);
    }

    const lastDot = (0, _lastIndexOf.default)(partial).call(partial, '.');
    let context = partial.substr(0, lastDot);
    let hint = partial.substr(lastDot + 1);

    if (lastDot === -1) {
      context = 'this';
      hint = partial;
    }

    this.events.request('console:executePartial', context, (err, result) => {
      var _context;

      if (err || !result) {
        return cb(null, [[], partial]);
      }

      let props = (0, _sort.default)(_context = (0, _getOwnPropertyNames.default)(result)).call(_context);

      if (hint !== "") {
        props = (0, _filter.default)(props).call(props, prop => {
          return (0, _indexOf.default)(prop).call(prop, hint) === 0;
        });
      }

      if (lastDot !== -1) {
        props = (0, _map.default)(props).call(props, prop => {
          return `${context}.${prop}`;
        });
      }

      if (props.length > 1) {
        console.log(props.join(', '));
      }

      cb(null, [props, partial]);
    });
  }

  start(done) {
    var _context2, _context3, _context4;

    this.replServer = repl.start({
      completer: (0, _bind.default)(_context2 = this.complete).call(_context2, this),
      prompt: "Embark (".cyan + this.env.green + ") > ".cyan,
      useGlobal: true,
      eval: (0, _bind.default)(_context3 = this.enhancedEval).call(_context3, this),
      writer: (0, _bind.default)(_context4 = this.enhancedWriter).call(_context4, this),
      input: this.inputStream,
      output: this.outputStream,
      terminal: true
    });
    this.events.request('console:history', (err, history) => {
      var _context5;

      (0, _forEach.default)(_context5 = history.split('\n')).call(_context5, cmd => {
        this.replServer.history.push(cmd);
      });
    });

    if (this.ipc.isServer()) {
      var _context6;

      this.events.on('console:history:save', (0, _bind.default)(_context6 = this.addHistory).call(_context6, this));
    } else if (this.ipc.connected) {
      var _context7;

      this.ipc.client.on('console:history:save', (0, _bind.default)(_context7 = this.addHistory).call(_context7, this));
    }

    this.events.request("runcode:getContext", (err, context) => {
      this.replServer.context = context;
    });
    this.replServer.on("exit", () => {
      process.exit(0);
    });
    done();
  }

}

module.exports = REPL;
//# sourceMappingURL=repl.js.map