"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));

var _find = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/find"));

var _parseInt2 = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/parse-int"));

var _embarkUtils = require("embark-utils");

const path = require('path');

const pkgUp = require('pkg-up');

let shelljs = require('shelljs');

class Simulator {
  constructor(options) {
    this.blockchainConfig = options.blockchainConfig;
    this.logger = options.logger;
  }
  /*eslint complexity: ["error", 26]*/


  run(options) {
    var _context;

    const cmds = [];

    const host = (0, _embarkUtils.dockerHostSwap)(options.host || this.blockchainConfig.rpcHost) || _embarkUtils.defaultHost;

    const configPort = this.blockchainConfig.wsRPC ? this.blockchainConfig.wsPort : this.blockchainConfig.rpcPort;
    let port = (0, _parseInt2.default)(options.port || configPort || 8545, 10);
    cmds.push("-p " + port);
    cmds.push("-h " + host);
    cmds.push("-l " + (options.gasLimit || this.blockchainConfig.targetGasLimit || 8000000)); // adding mnemonic only if it is defined in the blockchainConfig or options

    let mnemonicAccount = this.blockchainConfig.accounts ? (0, _find.default)(_context = this.blockchainConfig.accounts).call(_context, acc => acc.mnemonic) : {};
    mnemonicAccount = mnemonicAccount || {};
    const simulatorMnemonic = mnemonicAccount.mnemonic || options.simulatorMnemonic;

    if (simulatorMnemonic) {
      cmds.push("--mnemonic \"" + simulatorMnemonic + "\"");
    }

    cmds.push("-a " + (options.numAccounts || mnemonicAccount.numAddresses || 10));
    cmds.push("-e " + (options.defaultBalance || mnemonicAccount.balance || 100)); // as ganache-cli documentation explains, the simulatorAccounts configuration overrides a mnemonic

    let simulatorAccounts = this.blockchainConfig.simulatorAccounts || options.simulatorAccounts;

    if (simulatorAccounts && simulatorAccounts.length > 0) {
      let web3 = new (require('web3'))();
      let parsedAccounts;

      try {
        parsedAccounts = _embarkUtils.AccountParser.parseAccountsConfig(simulatorAccounts, web3, (0, _embarkUtils.dappPath)(), this.logger);
      } catch (e) {
        this.logger.error(e.message);
        process.exit(1);
      }

      (0, _forEach.default)(parsedAccounts).call(parsedAccounts, account => {
        let cmd = '--account="' + account.privateKey + ',' + account.hexBalance + '"';
        cmds.push(cmd);
      });
    } // adding blocktime only if it is defined in the blockchainConfig or options


    let simulatorBlocktime = this.blockchainConfig.simulatorBlocktime || options.simulatorBlocktime;

    if (simulatorBlocktime) {
      cmds.push("-b \"" + simulatorBlocktime + "\"");
    } // Setting up network id for simulator from blockchainConfig or options.
    // Otherwise ganache-cli would make random network id.


    let networkId = this.blockchainConfig.networkId || options.networkId;

    if (networkId) {
      // Don't handle networkId=="0" because it is not a valid networkId for ganache-cli.
      cmds.push("--networkId " + networkId);
    }

    this.runCommand(cmds, host, port);
  }

  runCommand(cmds) {
    const ganache_main = require.resolve('ganache-cli', {
      paths: [(0, _embarkUtils.embarkPath)('node_modules')]
    });

    const ganache_json = pkgUp.sync(path.dirname(ganache_main));
    const ganache_root = path.dirname(ganache_json);

    const ganache_bin = require(ganache_json).bin;

    let ganache;

    if (typeof ganache_bin === 'string') {
      ganache = path.join(ganache_root, ganache_bin);
    } else {
      ganache = path.join(ganache_root, ganache_bin['ganache-cli']);
    }

    const programName = 'ganache-cli';
    const program = ganache;
    this.logger.info(`running: ${programName} ${cmds.join(' ')}`);
    shelljs.exec(`node ${program} ${cmds.join(' ')}`, {
      async: true
    });
  }

}

module.exports = Simulator;
//# sourceMappingURL=simulator.js.map